% script file godunov.m
% Godunov's method for nonlinear equations
% 
% last changes: 03.05.03 by Vita
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%

EXAMPLE = 'A'; % 'A': Burger's eqn.
               % 'B': f(u) = u^4
	       % 'C': f(u) = u
INIT_EXAMPLE = 'B'; % 'A', 'B' or 'C'

Tmax = 5;

CFL = 1;	       
n = 81;	

% initial condition defined at grid points

if isequal(INIT_EXAMPLE,'A')
  ax = -10;
  bx = 10; 
  x = linspace(ax,bx,n);
  
  left = find(x<1);
  right = find(x>=1);
  u0 = zeros(n,1);
  u0(left) = -2;
  u0(right)= 1;
elseif isequal(INIT_EXAMPLE,'B')
  ax = -1;
  bx = 30; 
  x = linspace(ax,bx,n);
  
  left = find(x<0);
  right = find(x>=0);
  u0 = zeros(n,1);
  u0(left) = 2;
  u0(right)= -1; 
elseif isequal(INIT_EXAMPLE,'C')
  ax = 0;
  bx = 10; 
  x = linspace(ax,bx,n);
  
  u0 = sin(x);
else
  error('Only A, B, and C examples for i.c.!')
end
  
h = x(2)-x(1);

% find the maximum of f' to find the time step
u_min = min(u0);
u_max = max(u0);
u_temp = linspace(u_min,u_max,50);
flux_der_max = max(abs(flux_der(EXAMPLE,u_temp)));

k = CFL*h/flux_der_max;

lambda = k/h;

t = 0;

u = u0;
sonic_point = 0;
figure(1)

while t<Tmax
  t = t+k;
  
  fprim = flux_der(EXAMPLE,u);
  
  % the next can be also vectorized, but loop is easier
  % to understand
  
  u_new = zeros(n,1);
  % find u_star for each interval
  % periodicity can not be used here!!
  % the most trivial extension - by constant
  % u_star(i) corresponds to interval [i-1,i]
  u_star = zeros(n,1);
  u_star(1) = u(1); % assume, u(0) = u(1);
  
  for i=2:n
     if fprim(i-1)>=0 & fprim(i)>=0
      u_star(i) = u(i-1);
    elseif fprim(i-1)>=0 & fprim(i)<0
      s = (flux(EXAMPLE,u(i))-flux(EXAMPLE,u(i-1)))/(u(i)-u(i-1));
      if s>0
	u_star(i) = u(i-1);
      else
	u_star(i) = u(i);
      end
    elseif fprim(i-1)<0 & fprim(i)<0
      u_star(i) = u(i);
    else
      u_star(i)=sonic_point;
    end
  end
  
  % godunov update
  u_new = zeros(n,1);
  for i = 1:n-1
    u_new(i) = u(i) - lambda*(flux(EXAMPLE,u_star(i+1))-...
			     flux(EXAMPLE,u_star(i)));
  end
  % assume u(n+1) = u(n) => u_star(n+1) = u(n)
  u_new(n) = u(n) - lambda*(flux(EXAMPLE,u(n))-...
			   flux(EXAMPLE,u_star(n)));
  
  u = u_new;
  
  % VISUALIZE
  % by plotting remember that u(i) lives in the 
  % middle of the intervals
  P=plot(x,u,'r-');
  title(['t=',num2str(t)]);
  axis([ax bx min(u0) max(u0)]);
  set(P,'linewidth',3)
  drawnow
  %pause(0.01)
end

    
    
    
    
      

  
  
  